﻿namespace PaintShopSchedulingSimpleTypes
{
    /// <summary>
    /// Simple DTO to store information about a position in the schedule.
    /// </summary>
    public struct Position
    {
        /// <summary>
        /// The zero-indexed round location of the position
        /// </summary>
        public readonly int RoundIndex;

        /// <summary>
        /// The zero-indexed position of the cell within the round
        /// </summary>
        public readonly int PositionIndex;

        public Position(int roundIndex, int positionIndex)
        {
            RoundIndex = roundIndex;
            PositionIndex = positionIndex;
        }

        public override int GetHashCode()
        {
            return RoundIndex * 10000 + PositionIndex;
        }

        // We override equals and provide and IsEqual method here because of performance.
        // Standard equals will use reflection for comparison and is slow
        public override bool Equals(object obj)
        {
            if (!(obj is Position))
                return false;

            Position other = (Position)obj;
            return PositionIndex == other.PositionIndex && RoundIndex == other.RoundIndex;
        }
    }
}
