﻿namespace PaintShopSchedulingSimpleTypes
{
    public class SimpleInstanceSerializerFactory : ISimpleInstanceSerializerFactory
    {
        private static SimpleInstanceSerializerFactory _instance;
        public ISimpleInstanceSerializer DefaultSimpleInstanceSerializer { set; private get; }

        private SimpleInstanceSerializerFactory()
        {
            // Use standard serializer by default
            DefaultSimpleInstanceSerializer = new SimpleInstanceSerializer(new SimpleInstanceStringSerializer());
        }

        /// <summary>
        /// Get the singleton instance
        /// </summary>
        public static ISimpleInstanceSerializerFactory Instance
        {
            get
            {
                if (_instance == null)
                {
                    _instance = new SimpleInstanceSerializerFactory();
                }
                return _instance;
            }
        }

        /// <summary>
        /// Return the configured default instance serializer.  
        /// </summary>
        /// <returns>An instance for the configured default instance serializer.</returns>
        public ISimpleInstanceSerializer GetDefaultSimpleInstanceSerializer()
        {
            return DefaultSimpleInstanceSerializer;
        }
    }
}
