﻿using System.Collections.Generic;
using System.Linq;
using PaintShopSchedulingSimpleTypes;
using PaintShopSchedulingSimpleTypes.SimpleTypes;

namespace SimpleSolutionValidator.SolutionEvaluator
{
    public class CarrierAvailabilityEvaluator : ICarrierAvailabilityEvaluator
    {
        private int _totalViolations;
        private IList<int> _violationsPerRound;
        private ICollection<SimpleCarrierTypeInformation> _carrierTypeInformations;
        private readonly SimpleInstance _instance;

        public CarrierAvailabilityEvaluator(SimpleInstance instance)
        {
            _instance = instance;
        }

        public void Evaluate(ICollection<SimpleCarrierTypeInformation> carrierTypeInformations, ISimpleSolution solution)
        {
            _totalViolations = 0;
            _violationsPerRound = Enumerable.Repeat(0, solution.NumberOfRounds - 1).ToList();
            _carrierTypeInformations = carrierTypeInformations;

            for (int roundIndex = 1; roundIndex < solution.NumberOfRounds; roundIndex++) {
                foreach (int carrierTypeId in solution.GetDistinctCarrierTypeIdsInRound(roundIndex)) {
                    int numberOfUsedCarriers = solution.GetCarrierTypeIdsInRound(roundIndex).Count(x => x == carrierTypeId);
                    UpdateViolationsForCarrierInRound(carrierTypeId, numberOfUsedCarriers, roundIndex);
                }
            }
        }

        private void UpdateViolationsForCarrierInRound(int carrierTypeId, int numberOfUsedCarriers, int roundIndex)
        {
            int availableCarriers = 0;

            foreach (SimpleCarrierTypeInformation carrierInfo in _carrierTypeInformations) {
                if (carrierInfo.CarrierType.Id == carrierTypeId) {
                    availableCarriers = carrierInfo.Availabilities[roundIndex - 1];
                }
            }

            if (availableCarriers < numberOfUsedCarriers) {
                int violationsForCarrierInRound = numberOfUsedCarriers - availableCarriers;
                _totalViolations += violationsForCarrierInRound;
                System.Console.WriteLine("Too many carriers of type " + _instance.CarrierTypeNames[carrierTypeId] +
                                         " in round " + roundIndex + ". " + numberOfUsedCarriers +
                                         " scheduled but max availability is " + availableCarriers);
                _violationsPerRound[roundIndex - 1] = violationsForCarrierInRound;
            }
        }

        public int GetTotalViolations()
        {
            return _totalViolations;
        }

        public IList<int> GetViolationsPerRound()
        {
            return _violationsPerRound;
        }
    }
}