﻿using System.Collections.Generic;
using System.Linq;
using PaintShopSchedulingSimpleTypes;
using PaintShopSchedulingSimpleTypes.SimpleTypes;

namespace SimpleSolutionValidator.SolutionEvaluator
{
    public class ForbiddenCarrierTypeSequencesEvaluator : IForbiddenCarrierTypeSequencesEvaluator
    {
        private int _totalViolations;
        private IList<int> _totalViolationsPerRound;
        private ICollection<SimpleForbiddenCarrierTypeSequence> _forbiddenCarrierTypeSequences;
        private readonly SimpleInstance _instance;

        public ForbiddenCarrierTypeSequencesEvaluator(SimpleInstance instance)
        {
            _instance = instance;
        }

        public void Evaluate(ICollection<SimpleForbiddenCarrierTypeSequence> forbiddenCarrierTypeSequences, ISimpleSolution solution)
        {
            _totalViolations = 0;
            _totalViolationsPerRound = Enumerable.Repeat(0, solution.NumberOfRounds - 1).ToList();
            _forbiddenCarrierTypeSequences = forbiddenCarrierTypeSequences;

            for (int roundIndex = 1; roundIndex < solution.NumberOfRounds; roundIndex++)
            {
                // check for carrier type change to last carrier of previous round
                int numberOfCarriersInPrevRound = solution.GetNumberOfCarriersInRound(roundIndex - 1);
                int numberOfCarriersInCurrentRound = solution.GetNumberOfCarriersInRound(roundIndex);
                if (numberOfCarriersInPrevRound > 0 && numberOfCarriersInCurrentRound > 0)
                {
                    int fromCarrierId =
                        solution.GetCarrierTypeIdAtPosition(new Position(roundIndex - 1, numberOfCarriersInPrevRound - 1));
                    int toCarrierId = solution.GetCarrierTypeIdAtPosition(new Position(roundIndex, 0));

                    UpdateCarrierTypeChangeData(fromCarrierId, toCarrierId, roundIndex, 0);
                }

                for (int posIndex = 0; posIndex < numberOfCarriersInCurrentRound - 1; posIndex++) {

                    int fromCarrierId =
                        solution.GetCarrierTypeIdAtPosition(new Position(roundIndex, posIndex));
                    int toCarrierId = solution.GetCarrierTypeIdAtPosition(new Position(roundIndex, posIndex+1));

                    UpdateCarrierTypeChangeData(fromCarrierId, toCarrierId, roundIndex, posIndex+1);
                }
            }
        }

        private void UpdateCarrierTypeChangeData(int fromCarrierId, int toCarrierId, int roundIndex, int posIndex)
        {
            foreach (SimpleForbiddenCarrierTypeSequence forbiddenCarrierSeq in _forbiddenCarrierTypeSequences) {
                if (forbiddenCarrierSeq.FromCarrierType.Id == fromCarrierId &&
                    forbiddenCarrierSeq.ToCarrierType.Id == toCarrierId) {
                    _totalViolations++;
                    _totalViolationsPerRound[roundIndex - 1]++;

                    System.Console.WriteLine("Encountered forbidden carrier sequence in round " + roundIndex +
                                             " at position " + posIndex + ": Carrier of type " +
                                             _instance.CarrierTypeNames[toCarrierId] +
                                             " may not follow directly after type " +
                                             _instance.CarrierTypeNames[fromCarrierId]);
                }
            }


        }

        public int GetTotalViolations()
        {
            return _totalViolations;
        }

        public IList<int> GetViolationsPerRound()
        {
            return _totalViolationsPerRound;
        }
    }
}