﻿using System.Collections.Generic;
using PaintShopSchedulingSimpleTypes;
using PaintShopSchedulingSimpleTypes.SimpleTypes;

namespace SimpleSolutionValidator.SolutionEvaluator
{
    /// <summary>
    /// Evaluates a given solution regarding carrier availability violations
    /// </summary>
    public interface ICarrierAvailabilityEvaluator
    {
        /// <summary>
        /// Evaluates a given solution regarding availability violations.
        /// </summary>
        /// <param name="carrierTypeInformations">Collection containing carrier type informations, including availabilities per round.</param>
        /// <param name="solution">The solution that should be evaluated.</param>
        void Evaluate(ICollection<SimpleCarrierTypeInformation> carrierTypeInformations, ISimpleSolution solution);

        /// <summary>
        /// Gets the total violations from the last Evaluate call.
        /// </summary>
        int GetTotalViolations();

        /// <summary>
        /// Gets the violations per round from the last Evaluate call.
        /// </summary>
        IList<int> GetViolationsPerRound();
    }
}