﻿using System.Collections.Generic;
using PaintShopSchedulingSimpleTypes;
using PaintShopSchedulingSimpleTypes.SimpleTypes;

namespace SimpleSolutionValidator.SolutionEvaluator
{
    /// <summary>
    /// Evaluates color changes for a given candidate solution
    /// </summary>
    public interface IColorChangeEvaluator
    {
        /// <summary>
        /// Evaluate a given candidate solution for color change costs.
        /// </summary>
        /// <param name="colorTransitionCosts">Collection containing information about the color change costs.</param>
        /// <param name="solution">The solution that should be evaluated.</param>
        void Evaluate(ICollection<SimpleColorTransitionCost> colorTransitionCosts, ISimpleSolution solution);

        /// <summary>
        /// Get the total color changes from the last evaluated solution.
        /// </summary>
        int GetTotalColorChanges();

        /// <summary>
        /// Get the color changes per round from the last evaluated solution.
        /// A potential color change that occurs between the last carrier of the previous round and the first carrier of the current round will also be included in this value.
        /// </summary>
        IList<int> GetColorChangesPerRound();

        /// <summary>
        /// Get the total color change costs for the last evaluated solution.
        /// </summary>
        int GetTotalColorChangeCosts();

        /// <summary>
        /// Get the total color changes from the last evaluated solution.
        /// A potential color change that occurs between the last carrier of the previous round and the first carrier of the current round will also be included in this value.
        /// </summary>
        IList<int> GetColorCostsPerRound();

        /// <summary>
        /// Get the squared color costs per round, where the total costs are the sum of all squared costs per round.
        /// </summary>
        long GetSquaredColorCosts();

        /// <summary>
        /// Get the number of average color changes per round 
        /// </summary>
        double GetAverageColorChangesPerRound();

        /// <summary>
        /// Gets the maximum number of changes in a single round
        /// </summary>
        int GetMaximumColorChangesInSingleRound();

        /// <summary>
        /// Get the color change variance 
        /// </summary>
        double GetColorChangeVariance();
    }
}
