﻿using System.Collections.Generic;
using PaintShopSchedulingSimpleTypes;
using PaintShopSchedulingSimpleTypes.SimpleTypes;

namespace SimpleSolutionValidator.SolutionEvaluator
{
    /// <summary>
    /// Evaluates all unfulfilled demands for a given solution.
    /// </summary>
    public interface IUnfulfilledDemandEvaluator
    {
        /// <summary>
        /// Evaluates all unfulfilled demands for a given solution.
        /// </summary>
        /// <param name="demands">Collection containing all demands.</param>
        /// <param name="configurations">Collection containing all configuration information, including carried materials.</param>
        /// <param name="solution">Solution that should be evaluated.</param>
        void Evaluate(ICollection<SimpleDemand> demands, ICollection<SimpleCarrierConfiguration> configurations, ISimpleSolution solution);

        /// <summary>
        /// Gets the total number of unfulfilled demands.
        /// </summary>
        int GetUnfulfilledDemands();

        /// <summary>
        /// Gets the total number of missing demand pieces.
        /// </summary>
        int GetMissingDemandPieces();

        /// <summary>
        /// Gets the number of unfulfilled demands per round.
        /// </summary>
        IList<int> GetUnfulfilledDemandsPerRound();

        /// <summary>
        /// Gets the number of missing demand pieces per round.
        /// </summary>
        IList<int> GetMissingDemandPiecesPerRound();
    }
}
